<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2009 Snowflake Productions Gmbh (info@snowflake.ch)
*  All rights reserved
*
*  This script is part of the Typo3 project. The Typo3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

/* $Id: class.tx_t3blog_tcemain.php 61173 2012-04-27 12:45:29Z dmitry $ */

/**
 * This class implements a hook to TCEmain to ensure that data is correctly
 * inserted to pages when using TemplaVoila. It disables automatic TemplaVoila
 * element referencing for content elements that are part of t3blog post.
 */
class tx_t3blog_tcemain {

	/**
	 * Processes t3blog item deletion
	 *
	 * @param string $command
	 * @param string $table
	 * @param mixed $id
	 * @return void
	 */
	public function processCmdmap_postProcess($command, $table, $id) {
		if ($table == 'tx_t3blog_post' && $command == 'delete') {
			$this->deletePostData(intval($id));
		}
	}

	/**
	 * Checks if TemplaVoila references should be disabled for this record
	 *
	 * @param string $status
	 * @param string $table
	 * @param int $id
	 * @param array $fieldArray
	 * @param t3lib_TCEmain $pObj
	 * @see t3lib_TCEmain::process_datamap()
	 * @see tx_templavoila_tcemain::processDatamap_afterDatabaseOperations()
	 */
	public function processDatamap_preProcessFieldArray(array $incomingFieldArray, $table, $id, t3lib_TCEmain &$pObj) {
		if (t3lib_extMgm::isLoaded('templavoila' &&
				!isset($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['tx_templavoila_tcemain']['doNotInsertElementRefsToPage'])) ||
				!$GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['tx_templavoila_tcemain']['doNotInsertElementRefsToPage']) {
			$disableReferencing = false;
			if ($table == 'tx_t3blog_post') {
				$disableReferencing = true;
			}
			elseif ($table == 'tt_content' && $this->isT3BlogPage($this->getPid($incomingFieldArray, $id))) {
				$disableReferencing = true;
			}
			if ($disableReferencing) {
				$GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['tx_templavoila_tcemain']['doNotInsertElementRefsToPage'] = true;
			}
		}
	}

	/**
	 * Deletes all data associated with the post when post is deleted
	 *
	 * @param int $id
	 * @return void
	 */
	protected function deletePostData($id) {
		$command = array(
			'tx_t3blog_com' => $this->getDeleteArrayForTable($id, 'tx_t3blog_com', 'fk_post'),
			'tx_t3blog_com_nl' => $this->getDeleteArrayForTable($id, 'tx_t3blog_com_nl', 'post_uid'),
			'tx_t3blog_trackback' => $this->getDeleteArrayForTable($id, 'tx_t3blog_trackback', 'postid'),
			'tt_content' => $this->getDeleteArrayForTable($id, 'tt_content', 'irre_parentid', ' AND irre_parenttable=\'tx_t3blog_post\'')
		);
		$tce = t3lib_div::makeInstance('t3lib_TCEmain');
		/* @var $tce t3lib_TCEmain */
		$tce->start(array(), $command);
		$tce->process_cmdmap();
	}

	protected function getDeleteArrayForTable($postId, $tableName, $fieldName, $extraWhere = '') {
		$command = array();
		$where = $fieldName . '=' . $postId . t3lib_BEfunc::deleteClause($tableName) . $extraWhere;
		$data = $GLOBALS['TYPO3_DB']->exec_SELECTgetRows('uid', $tableName, $where);
		foreach ($data as $record) {
			$command[$record['uid']]['delete'] = 1;
		}
		return $command;
	}

	/**
	 * Obtains page if for the record
	 *
	 * @param array $incomingFieldArray
	 * @param mixed $recordId
	 * @return void
	 */
	protected function getPid(array $incomingFieldArray, $recordId) {
		$pid = 0;
		if (isset($incomingFieldArray['pid']) &&
				t3lib_div::testInt($incomingFieldArray['pid']) &&
				$incomingFieldArray['pid'] > 0) {
			$pid = $incomingFieldArray['pid'];
		}
		elseif (t3lib_div::testInt($recordId) && $recordId > 0) {
			$record = t3lib_BEfunc::getRecord('tt_content', $recordId,  'pid');
			$pid = $record['pid'];
		}
		return $pid;
	}

	/**
	 * Checks if this page is a t3blog page
	 *
	 * @return boolean
	 */
	protected function isT3BlogPage($pageId) {
		$pageRecord = t3lib_BEfunc::getRecord('pages', $pageId, 'module');
		return (is_array($pageRecord) && $pageRecord['module'] == 't3blog');
	}
}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/t3blog/hooks/class.tx_t3blog_tcemain.php'])    {
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/t3blog/hooks/class.tx_t3blog_tcemain.php']);
}

?>